@@ -24,10 +24,12 @@
 
 #include "../../core/windows/SDL_windows.h"
 
+#include "SDL_log.h"
 #include "../SDL_sysvideo.h"
 #include "../SDL_pixels_c.h"
 #include "../../events/SDL_keyboard_c.h"
 #include "../../events/SDL_mouse_c.h"
+#include "../../events/SDL_windowevents_c.h"
 #include "../../SDL_hints_c.h"
 
 #include "SDL_windowsvideo.h"
@@ -46,6 +48,8 @@
 #define SWP_NOCOPYBITS 0
 #endif
 
+/* #define HIGHDPI_DEBUG */
+
 /* Fake window to help with DirectInput events. */
 HWND SDL_HelperWindow = NULL;
 static const TCHAR *SDL_HelperWindowClassName = TEXT("SDLHelperWindowInputCatcher");
@@ -113,26 +117,95 @@ GetWindowStyle(SDL_Window * window)
     return style;
 }
 
+/**
+ * Returns arguments to pass to SetWindowPos - the window rect, including frame, in Windows coordinates.
+ * Can be called before we have a HWND.
+ */
 static void
 WIN_AdjustWindowRectWithStyle(SDL_Window *window, DWORD style, BOOL menu, int *x, int *y, int *width, int *height, SDL_bool use_current)
 {
+    SDL_VideoData* videodata = SDL_GetVideoDevice() ? SDL_GetVideoDevice()->driverdata : NULL;
     RECT rect;
+    int dpi = 96;
+#if !defined(__XBOXONE__) && !defined(__XBOXSERIES__)
+    UINT frame_dpi;
+#endif
 
+    /* Client rect, in SDL screen coordinates */
+    *x = (use_current ? window->x : window->windowed.x);
+    *y = (use_current ? window->y : window->windowed.y);
+    *width = (use_current ? window->w : window->windowed.w);
+    *height = (use_current ? window->h : window->windowed.h);
+
+    /* Convert client rect from SDL coordinates to pixels (no-op if DPI scaling not enabled) */
+#if !defined(__XBOXONE__) && !defined(__XBOXSERIES__)
+    WIN_ScreenPointFromSDL(x, y, &dpi);
+#endif
+    /* Note, use the guessed DPI returned from WIN_ScreenPointFromSDL rather than the cached one in
+       data->scaling_dpi.
+
+       - This is called before the window is created, so we can't rely on data->scaling_dpi
+       - Bug workaround: when leaving exclusive fullscreen, the cached DPI and window DPI reported
+         by GetDpiForWindow will be wrong, and would cause windows shrinking slightly when
+         going from exclusive fullscreen to windowed on a HighDPI monitor with scaling if we used them.
+    */
+    *width = MulDiv(*width, dpi, 96);
+    *height = MulDiv(*height, dpi, 96);
+
+    /* Copy the client size in pixels into this rect structure,
+       which we'll then adjust with AdjustWindowRectEx */
     rect.left = 0;
     rect.top = 0;
-    rect.right = (use_current ? window->w : window->windowed.w);
-    rect.bottom = (use_current ? window->h : window->windowed.h);
+    rect.right = *width;
+    rect.bottom = *height;
 
     /* borderless windows will have WM_NCCALCSIZE return 0 for the non-client area. When this happens, it looks like windows will send a resize message
        expanding the window client area to the previous window + chrome size, so shouldn't need to adjust the window size for the set styles.
      */
-    if (!(window->flags & SDL_WINDOW_BORDERLESS))
+    if (!(window->flags & SDL_WINDOW_BORDERLESS)) {
+#if defined(__XBOXONE__) || defined(__XBOXSERIES__)
         AdjustWindowRectEx(&rect, style, menu, 0);
+#else
+        if (WIN_IsPerMonitorV2DPIAware(SDL_GetVideoDevice())) {
+            /* With per-monitor v2, the window border/titlebar size depend on the DPI, so we need to call AdjustWindowRectExForDpi instead of 
+               AdjustWindowRectEx. */
+            UINT unused;
+            RECT screen_rect;
+            HMONITOR mon;
 
-    *x = (use_current ? window->x : window->windowed.x) + rect.left;
-    *y = (use_current ? window->y : window->windowed.y) + rect.top;
+            screen_rect.left = *x;
+            screen_rect.top = *y;
+            screen_rect.right = *x + *width;
+            screen_rect.bottom  = *y + *height;
+
+            mon = MonitorFromRect(&screen_rect, MONITOR_DEFAULTTONEAREST);
+
+            /* GetDpiForMonitor docs promise to return the same hdpi / vdpi */
+            if (videodata->GetDpiForMonitor(mon, MDT_EFFECTIVE_DPI, &frame_dpi, &unused) != S_OK) {
+                frame_dpi = 96;
+            }
+
+            videodata->AdjustWindowRectExForDpi(&rect, style, menu, 0, frame_dpi);
+        } else {
+            AdjustWindowRectEx(&rect, style, menu, 0);
+        }  
+#endif
+    }
+
+    /* Final rect in Windows screen space, including the frame */
+    *x += rect.left;
+    *y += rect.top;
     *width = (rect.right - rect.left);
     *height = (rect.bottom - rect.top);
+
+#ifdef HIGHDPI_DEBUG
+    SDL_Log("WIN_AdjustWindowRectWithStyle: in: %d, %d, %dx%d, returning: %d, %d, %dx%d, used dpi %d for frame calculation", 
+        (use_current ? window->x : window->windowed.x),
+        (use_current ? window->y : window->windowed.y),
+        (use_current ? window->w : window->windowed.w),
+        (use_current ? window->h : window->windowed.h),
+        *x, *y, *width, *height, frame_dpi);
+#endif
 }
 
 static void
@@ -144,7 +217,11 @@ WIN_AdjustWindowRect(SDL_Window *window, int *x, int *y, int *width, int *height
     BOOL menu;
 
     style = GetWindowLong(hwnd, GWL_STYLE);
+#if defined(__XBOXONE__) || defined(__XBOXSERIES__)
+    menu = FALSE;
+#else
     menu = (style & WS_CHILDWINDOW) ? FALSE : (GetMenu(hwnd) != NULL);
+#endif
     WIN_AdjustWindowRectWithStyle(window, style, menu, x, y, width, height, use_current);
 }
 
@@ -178,6 +255,47 @@ WIN_MouseRelativeModeCenterChanged(void *userdata, const char *name, const char
     data->mouse_relative_mode_center = SDL_GetStringBoolean(hint, SDL_TRUE);
 }
 
+static int
+WIN_GetScalingDPIForHWND(const SDL_VideoData *videodata, HWND hwnd)
+{
+#if defined(__XBOXONE__) || defined(__XBOXSERIES__)
+    return 96;
+#else
+    /* DPI scaling not requested? */
+    if (!videodata->dpi_scaling_enabled) {
+        return 96;
+    }
+
+    /* Window 10+ */
+    if (videodata->GetDpiForWindow) {
+        return videodata->GetDpiForWindow(hwnd);
+    }
+
+    /* Window 8.1+ */
+    if (videodata->GetDpiForMonitor) {
+        HMONITOR monitor = MonitorFromWindow(hwnd, MONITOR_DEFAULTTONEAREST);
+        if (monitor) {
+            UINT dpi_uint, unused;
+            if (S_OK == videodata->GetDpiForMonitor(monitor, MDT_EFFECTIVE_DPI, &dpi_uint, &unused)) {
+                return (int)dpi_uint;
+            }
+        }
+        return 96;
+    }
+
+    /* Windows Vista-8.0 */
+    {
+        HDC hdc = GetDC(NULL);
+        if (hdc) {
+            int dpi = GetDeviceCaps(hdc, LOGPIXELSX);
+            ReleaseDC(NULL, hdc);
+            return dpi;
+        }
+        return 96;
+    }
+#endif
+}
+
 static int
 SetupWindowData(_THIS, SDL_Window * window, HWND hwnd, HWND parent, SDL_bool created)
 {
@@ -192,7 +310,9 @@ SetupWindowData(_THIS, SDL_Window * window, HWND hwnd, HWND parent, SDL_bool cre
     data->window = window;
     data->hwnd = hwnd;
     data->parent = parent;
+#if !defined(__XBOXONE__) && !defined(__XBOXSERIES__) 
     data->hdc = GetDC(hwnd);
+#endif
     data->hinstance = (HINSTANCE) GetWindowLongPtr(hwnd, GWLP_HINSTANCE);
     data->created = created;
     data->high_surrogate = 0;
@@ -200,17 +320,24 @@ SetupWindowData(_THIS, SDL_Window * window, HWND hwnd, HWND parent, SDL_bool cre
     data->last_pointer_update = (LPARAM)-1;
     data->videodata = videodata;
     data->initializing = SDL_TRUE;
+    data->scaling_dpi = WIN_GetScalingDPIForHWND(videodata, hwnd);
+
+#ifdef HIGHDPI_DEBUG
+    SDL_Log("SetupWindowData: initialized data->scaling_dpi to %d", data->scaling_dpi);
+#endif
 
     SDL_AddHintCallback(SDL_HINT_MOUSE_RELATIVE_MODE_CENTER, WIN_MouseRelativeModeCenterChanged, data);
 
     window->driverdata = data;
 
+#if !defined(__XBOXONE__) && !defined(__XBOXSERIES__) 
     /* Associate the data with the window */
     if (!SetProp(hwnd, TEXT("SDL_WindowData"), data)) {
         ReleaseDC(hwnd, data->hdc);
         SDL_free(data);
         return WIN_SetError("SetProp() failed");
     }
+#endif
 
     /* Set up the window proc function */
 #ifdef GWLP_WNDPROC
@@ -235,27 +362,37 @@ SetupWindowData(_THIS, SDL_Window * window, HWND hwnd, HWND parent, SDL_bool cre
         if (GetClientRect(hwnd, &rect)) {
             int w = rect.right;
             int h = rect.bottom;
+
+            WIN_ClientPointToSDL(window, &w, &h);
             if ((window->windowed.w && window->windowed.w != w) || (window->windowed.h && window->windowed.h != h)) {
                 /* We tried to create a window larger than the desktop and Windows didn't allow it.  Override! */
                 int x, y;
                 /* Figure out what the window area will be */
                 WIN_AdjustWindowRect(window, &x, &y, &w, &h, SDL_FALSE);
+                data->expected_resize = SDL_TRUE;
                 SetWindowPos(hwnd, HWND_NOTOPMOST, x, y, w, h, SWP_NOCOPYBITS | SWP_NOZORDER | SWP_NOACTIVATE);
+                data->expected_resize = SDL_FALSE;
             } else {
                 window->w = w;
                 window->h = h;
             }
         }
     }
+#if !defined(__XBOXONE__) && !defined(__XBOXSERIES__) 
     {
         POINT point;
         point.x = 0;
         point.y = 0;
         if (ClientToScreen(hwnd, &point)) {
-            window->x = point.x;
-            window->y = point.y;
+            int x = point.x;
+            int y = point.y;
+            WIN_ScreenPointToSDL(&x, &y);
+            window->x = x;
+            window->y = y;
         }
     }
+    WIN_UpdateWindowICCProfile(window, SDL_FALSE);
+#endif
     {
         DWORD style = GetWindowLong(hwnd, GWL_STYLE);
         if (style & WS_VISIBLE) {
@@ -290,16 +427,27 @@ SetupWindowData(_THIS, SDL_Window * window, HWND hwnd, HWND parent, SDL_bool cre
             window->flags &= ~SDL_WINDOW_MINIMIZED;
         }
     }
+#if defined(__XBOXONE__) || defined(__XBOXSERIES__)
+    window->flags |= SDL_WINDOW_INPUT_FOCUS;
+#else
     if (GetFocus() == hwnd) {
         window->flags |= SDL_WINDOW_INPUT_FOCUS;
         SDL_SetKeyboardFocus(window);
         WIN_UpdateClipCursor(window);
     }
+#endif
 
+#if !defined(__XBOXONE__) && !defined(__XBOXSERIES__)
     /* Enable multi-touch */
     if (videodata->RegisterTouchWindow) {
         videodata->RegisterTouchWindow(hwnd, (TWF_FINETOUCH|TWF_WANTPALM));
     }
+#endif
+
+    /* Force the SDL_WINDOW_ALLOW_HIGHDPI window flag if we are doing DPI scaling */
+    if (videodata->dpi_scaling_enabled) {
+        window->flags |= SDL_WINDOW_ALLOW_HIGHDPI;
+    }
 
     data->initializing = SDL_FALSE;
 
@@ -314,11 +462,16 @@ static void CleanupWindowData(_THIS, SDL_Window * window)
     if (data) {
         SDL_DelHintCallback(SDL_HINT_MOUSE_RELATIVE_MODE_CENTER, WIN_MouseRelativeModeCenterChanged, data);
 
+#if !defined(__XBOXONE__) && !defined(__XBOXSERIES__)
+        if (data->ICMFileName) {
+            SDL_free(data->ICMFileName);
+        }
         if (data->keyboard_hook) {
             UnhookWindowsHookEx(data->keyboard_hook);
         }
         ReleaseDC(data->hwnd, data->hdc);
         RemoveProp(data->hwnd, TEXT("SDL_WindowData"));
+#endif
         if (data->created) {
             DestroyWindow(data->hwnd);
             if (data->parent) {
@@ -420,6 +573,9 @@ WIN_CreateWindow(_THIS, SDL_Window * window)
 int
 WIN_CreateWindowFrom(_THIS, SDL_Window * window, const void *data)
 {
+#if defined(__XBOXONE__) || defined(__XBOXSERIES__)
+    return -1;
+#else
     HWND hwnd = (HWND) data;
     LPTSTR title;
     int titleLen;
@@ -471,20 +627,24 @@ WIN_CreateWindowFrom(_THIS, SDL_Window * window, const void *data)
     }
 #endif
     return 0;
+#endif /*!defined(__XBOXONE__) && !defined(__XBOXSERIES__)*/
 }
 
 void
 WIN_SetWindowTitle(_THIS, SDL_Window * window)
 {
+#if !defined(__XBOXONE__) && !defined(__XBOXSERIES__)
     HWND hwnd = ((SDL_WindowData *) window->driverdata)->hwnd;
     LPTSTR title = WIN_UTF8ToString(window->title);
     SetWindowText(hwnd, title);
     SDL_free(title);
+#endif
 }
 
 void
 WIN_SetWindowIcon(_THIS, SDL_Window * window, SDL_Surface * icon)
 {
+#if !defined(__XBOXONE__) && !defined(__XBOXSERIES__)
     HWND hwnd = ((SDL_WindowData *) window->driverdata)->hwnd;
     HICON hicon = NULL;
     BYTE *icon_bmp;
@@ -536,12 +696,16 @@ WIN_SetWindowIcon(_THIS, SDL_Window * window, SDL_Surface * icon)
 
     /* Set the icon in the task manager (should we do this?) */
     SendMessage(hwnd, WM_SETICON, ICON_BIG, (LPARAM) hicon);
+#endif
 }
 
 void
 WIN_SetWindowPosition(_THIS, SDL_Window * window)
 {
-    WIN_SetWindowPositionInternal(_this, window, SWP_NOCOPYBITS | SWP_NOSIZE | SWP_NOACTIVATE);
+    /* HighDPI support: removed SWP_NOSIZE. If the move results in a DPI change, we need to allow
+     * the window to resize (e.g. AdjustWindowRectExForDpi frame sizes are different).
+     */
+    WIN_SetWindowPositionInternal(_this, window, SWP_NOCOPYBITS | SWP_NOACTIVATE);
 }
 
 void
@@ -553,6 +717,21 @@ WIN_SetWindowSize(_THIS, SDL_Window * window)
 int
 WIN_GetWindowBordersSize(_THIS, SDL_Window * window, int *top, int *left, int *bottom, int *right)
 {
+#if defined(__XBOXONE__) || defined(__XBOXSERIES__)
+    HWND hwnd = ((SDL_WindowData *) window->driverdata)->hwnd;
+    RECT rcClient;
+
+    /* rcClient stores the size of the inner window, while rcWindow stores the outer size relative to the top-left
+     * screen position; so the top/left values of rcClient are always {0,0} and bottom/right are {height,width} */
+    GetClientRect(hwnd, &rcClient);
+
+    *top = rcClient.top;
+    *left = rcClient.left;
+    *bottom = rcClient.bottom;
+    *right = rcClient.right;
+
+    return 0;
+#else /*!defined(__XBOXONE__) && !defined(__XBOXSERIES__)*/
     HWND hwnd = ((SDL_WindowData *) window->driverdata)->hwnd;
     RECT rcClient, rcWindow;
     POINT ptDiff;
@@ -591,6 +770,7 @@ WIN_GetWindowBordersSize(_THIS, SDL_Window * window, int *top, int *left, int *b
     *right  = rcWindow.right  - rcClient.right;
 
     return 0;
+#endif /*!defined(__XBOXONE__) && !defined(__XBOXSERIES__)*/
 }
 
 void
@@ -619,6 +799,7 @@ WIN_HideWindow(_THIS, SDL_Window * window)
 void
 WIN_RaiseWindow(_THIS, SDL_Window * window)
 {
+#if !defined(__XBOXONE__) && !defined(__XBOXSERIES__)
     /* If desired, raise the window more forcefully.
      * Technique taken from http://stackoverflow.com/questions/916259/ .
      * Specifically, http://stackoverflow.com/a/34414846 .
@@ -651,6 +832,7 @@ WIN_RaiseWindow(_THIS, SDL_Window * window)
         SetFocus(hwnd);
         SetActiveWindow(hwnd);
     }
+#endif /*!defined(__XBOXONE__) && !defined(__XBOXSERIES__)*/
 }
 
 void
@@ -723,12 +905,17 @@ WIN_RestoreWindow(_THIS, SDL_Window * window)
     data->expected_resize = SDL_FALSE;
 }
 
+/**
+ * Reconfigures the window to fill the given display, if fullscreen is true, otherwise restores the window.
+ */
 void
 WIN_SetWindowFullscreen(_THIS, SDL_Window * window, SDL_VideoDisplay * display, SDL_bool fullscreen)
 {
+#if !defined(__XBOXONE__) && !defined(__XBOXSERIES__) 
+    SDL_DisplayData *displaydata = (SDL_DisplayData *) display->driverdata;
     SDL_WindowData *data = (SDL_WindowData *) window->driverdata;
     HWND hwnd = data->hwnd;
-    SDL_Rect bounds;
+    MONITORINFO minfo;
     DWORD style;
     HWND top;
     int x, y;
@@ -741,6 +928,10 @@ WIN_SetWindowFullscreen(_THIS, SDL_Window * window, SDL_VideoDisplay * display,
         return;
     }
 
+#ifdef HIGHDPI_DEBUG
+    SDL_Log("WIN_SetWindowFullscreen: %d", (int)fullscreen);
+#endif
+
     if (SDL_ShouldAllowTopmost() && (window->flags & SDL_WINDOW_ALWAYS_ON_TOP)) {
         top = HWND_TOPMOST;
     } else {
@@ -751,13 +942,20 @@ WIN_SetWindowFullscreen(_THIS, SDL_Window * window, SDL_VideoDisplay * display,
     style &= ~STYLE_MASK;
     style |= GetWindowStyle(window);
 
-    WIN_GetDisplayBounds(_this, display, &bounds);
+    /* Use GetMonitorInfo instead of WIN_GetDisplayBounds because we want the
+       monitor bounds in Windows coordinates (pixels) rather than SDL coordinates (points). */
+    SDL_zero(minfo);
+    minfo.cbSize = sizeof(MONITORINFO);
+    if (!GetMonitorInfo(displaydata->MonitorHandle, &minfo)) {
+        SDL_SetError("GetMonitorInfo failed");
+        return;
+    }
 
     if (fullscreen) {
-        x = bounds.x;
-        y = bounds.y;
-        w = bounds.w;
-        h = bounds.h;
+        x = minfo.rcMonitor.left;
+        y = minfo.rcMonitor.top;
+        w = minfo.rcMonitor.right - minfo.rcMonitor.left;
+        h = minfo.rcMonitor.bottom - minfo.rcMonitor.top;
 
         /* Unset the maximized flag.  This fixes
            https://bugzilla.libsdl.org/show_bug.cgi?id=3215
@@ -787,8 +985,15 @@ WIN_SetWindowFullscreen(_THIS, SDL_Window * window, SDL_VideoDisplay * display,
     data->expected_resize = SDL_TRUE;
     SetWindowPos(hwnd, top, x, y, w, h, SWP_NOCOPYBITS | SWP_NOACTIVATE);
     data->expected_resize = SDL_FALSE;
+
+#ifdef HIGHDPI_DEBUG
+    SDL_Log("WIN_SetWindowFullscreen: %d finished. Set window to %d,%d, %dx%d", (int)fullscreen, x, y, w, h);
+#endif
+
+#endif /*!defined(__XBOXONE__) && !defined(__XBOXSERIES__)*/
 }
 
+#if !defined(__XBOXONE__) && !defined(__XBOXSERIES__)
 int
 WIN_SetWindowGammaRamp(_THIS, SDL_Window * window, const Uint16 * ramp)
 {
@@ -808,29 +1013,53 @@ WIN_SetWindowGammaRamp(_THIS, SDL_Window * window, const Uint16 * ramp)
     return succeeded ? 0 : -1;
 }
 
-void*
+void
+WIN_UpdateWindowICCProfile(SDL_Window * window, SDL_bool send_event)
+{
+    SDL_WindowData *data = (SDL_WindowData *) window->driverdata;
+    SDL_VideoDisplay *display = SDL_GetDisplayForWindow(window);
+    SDL_DisplayData *displaydata = display ? (SDL_DisplayData*)display->driverdata : NULL;
+
+    if (displaydata) {
+        HDC hdc = CreateDCW(displaydata->DeviceName, NULL, NULL, NULL);
+        if (hdc) {
+            WCHAR fileName[MAX_PATH];
+            DWORD fileNameSize = SDL_arraysize(fileName);
+            if (GetICMProfileW(hdc, &fileNameSize, fileName)) {
+                /* fileNameSize includes '\0' on return */
+                if (!data->ICMFileName ||
+                    SDL_wcscmp(data->ICMFileName, fileName) != 0) {
+                    if (data->ICMFileName) {
+                        SDL_free(data->ICMFileName);
+                    }
+                    data->ICMFileName = SDL_wcsdup(fileName);
+                    if (send_event) {
+                        SDL_SendWindowEvent(window, SDL_WINDOWEVENT_ICCPROF_CHANGED, 0, 0);
+                    }
+                }
+            }
+            DeleteDC(hdc);
+        }
+    }
+}
+
+void *
 WIN_GetWindowICCProfile(_THIS, SDL_Window * window, size_t * size)
 {
-    SDL_VideoDisplay* display = SDL_GetDisplayForWindow(window);
-    SDL_DisplayData* data = (SDL_DisplayData*)display->driverdata;
-    HDC hdc;
-    BOOL succeeded = FALSE;
-    WCHAR filename[MAX_PATH];
-    DWORD fileNameSize = MAX_PATH;
-    void* iccProfileData = NULL;
+    SDL_WindowData *data = (SDL_WindowData *) window->driverdata;
+    char *filename_utf8;
+    void *iccProfileData = NULL;
 
-    hdc = CreateDCW(data->DeviceName, NULL, NULL, NULL);
-    if (hdc) {
-        succeeded = GetICMProfileW(hdc, &fileNameSize, filename);
-        DeleteDC(hdc);
-    }
-
-    if (succeeded) {
-        iccProfileData = SDL_LoadFile(WIN_StringToUTF8(filename), size);
-        if (!iccProfileData)
+    filename_utf8 = WIN_StringToUTF8(data->ICMFileName);
+    if (filename_utf8) {
+        iccProfileData = SDL_LoadFile(filename_utf8, size);
+        if (!iccProfileData) {
             SDL_SetError("Could not open ICC profile");
+        }
+        SDL_free(filename_utf8);
+    } else {
+        SDL_OutOfMemory();
     }
-
     return iccProfileData;
 }
 
@@ -918,6 +1147,7 @@ WIN_SetWindowKeyboardGrab(_THIS, SDL_Window * window, SDL_bool grabbed)
         WIN_UngrabKeyboard(window);
     }
 }
+#endif /*!defined(__XBOXONE__) && !defined(__XBOXSERIES__)*/
 
 void
 WIN_DestroyWindow(_THIS, SDL_Window * window)
@@ -945,8 +1175,8 @@ WIN_GetWindowWMInfo(_THIS, SDL_Window * window, SDL_SysWMinfo * info)
 
         return SDL_TRUE;
     } else {
-        SDL_SetError("Application not compiled with SDL %d.%d",
-                     SDL_MAJOR_VERSION, SDL_MINOR_VERSION);
+        SDL_SetError("Application not compiled with SDL %d",
+                     SDL_MAJOR_VERSION);
         return SDL_FALSE;
     }
 }
@@ -1020,6 +1250,7 @@ SDL_HelperWindowDestroy(void)
     }
 }
 
+#if !defined(__XBOXONE__) && !defined(__XBOXSERIES__)
 void WIN_OnWindowEnter(_THIS, SDL_Window * window)
 {
     SDL_WindowData *data = (SDL_WindowData *) window->driverdata;
@@ -1078,12 +1309,19 @@ WIN_UpdateClipCursor(SDL_Window *window)
                 ClientToScreen(data->hwnd, (LPPOINT) & rect);
                 ClientToScreen(data->hwnd, (LPPOINT) & rect + 1);
                 if (window->mouse_rect.w > 0 && window->mouse_rect.h > 0) {
+                    SDL_Rect mouse_rect_win_client;
                     RECT mouse_rect, intersection;
 
-                    mouse_rect.left = rect.left + window->mouse_rect.x;
-                    mouse_rect.top = rect.top + window->mouse_rect.y;
-                    mouse_rect.right = mouse_rect.left + window->mouse_rect.w - 1;
-                    mouse_rect.bottom = mouse_rect.top + window->mouse_rect.h - 1;
+                    /* mouse_rect_win_client is the mouse rect in Windows client space */
+                    mouse_rect_win_client = window->mouse_rect;
+                    WIN_ClientPointFromSDL(window, &mouse_rect_win_client.x, &mouse_rect_win_client.y);
+                    WIN_ClientPointFromSDL(window, &mouse_rect_win_client.w, &mouse_rect_win_client.h);
+
+                    /* mouse_rect is the rect in Windows screen space */
+                    mouse_rect.left = rect.left + mouse_rect_win_client.x;
+                    mouse_rect.top = rect.top + mouse_rect_win_client.y;
+                    mouse_rect.right = mouse_rect.left + mouse_rect_win_client.w;
+                    mouse_rect.bottom = mouse_rect.top + mouse_rect_win_client.h;
                     if (IntersectRect(&intersection, &rect, &mouse_rect)) {
                         SDL_memcpy(&rect, &intersection, sizeof(rect));
                     } else if ((window->flags & SDL_WINDOW_MOUSE_GRABBED) != 0) {
@@ -1125,10 +1363,14 @@ WIN_SetWindowHitTest(SDL_Window *window, SDL_bool enabled)
 {
     return 0;  /* just succeed, the real work is done elsewhere. */
 }
+#endif /*!defined(__XBOXONE__) && !defined(__XBOXSERIES__)*/
 
 int
 WIN_SetWindowOpacity(_THIS, SDL_Window * window, float opacity)
 {
+#if defined(__XBOXONE__) || defined(__XBOXSERIES__)
+    return -1;
+#else
     const SDL_WindowData *data = (SDL_WindowData *) window->driverdata;
     const HWND hwnd = data->hwnd;
     const LONG style = GetWindowLong(hwnd, GWL_EXSTYLE);
@@ -1157,8 +1399,71 @@ WIN_SetWindowOpacity(_THIS, SDL_Window * window, float opacity)
     }
 
     return 0;
+#endif /*!defined(__XBOXONE__) && !defined(__XBOXSERIES__)*/
 }
 
+/**
+ * Returns the drawable size in pixels (GetClientRect).
+ */
+void
+WIN_GetDrawableSize(const SDL_Window *window, int *w, int *h)
+{
+    HWND hwnd;
+    RECT rect;
+
+    if (!window || !(window->driverdata)) {
+        *w = 0;
+        *h = 0;
+        return;
+    }
+
+    hwnd = ((SDL_WindowData *)window->driverdata)->hwnd;
+    if (GetClientRect(hwnd, &rect)) {
+        *w = rect.right;
+        *h = rect.bottom;
+    } else {
+        *w = 0;
+        *h = 0;
+    }
+}
+
+/**
+ * Convert a point in the client area from pixels to DPI-scaled points.
+ * 
+ * No-op if DPI scaling is not enabled.
+ */
+void
+WIN_ClientPointToSDL(const SDL_Window *window, int *x, int *y)
+{
+    const SDL_WindowData *data = ((SDL_WindowData *)window->driverdata);
+    const SDL_VideoData *videodata = data->videodata;
+
+    if (!videodata->dpi_scaling_enabled)
+        return;
+
+    *x = MulDiv(*x, 96, data->scaling_dpi);
+    *y = MulDiv(*y, 96, data->scaling_dpi);
+}
+
+/**
+ * Convert a point in the client area from DPI-scaled points to pixels.
+ * 
+ * No-op if DPI scaling is not enabled.
+ */
+void
+WIN_ClientPointFromSDL(const SDL_Window *window, int *x, int *y)
+{
+    const SDL_WindowData *data = ((SDL_WindowData *)window->driverdata);
+    const SDL_VideoData *videodata = data->videodata;
+
+    if (!videodata->dpi_scaling_enabled)
+        return;
+    
+    *x = MulDiv(*x, data->scaling_dpi, 96);
+    *y = MulDiv(*y, data->scaling_dpi, 96);
+}
+
+#if !defined(__XBOXONE__) && !defined(__XBOXSERIES__)
 void
 WIN_AcceptDragAndDrop(SDL_Window * window, SDL_bool accept)
 {
@@ -1193,6 +1498,7 @@ WIN_FlashWindow(_THIS, SDL_Window * window, SDL_FlashOperation operation)
 
     return 0;
 }
+#endif /*!defined(__XBOXONE__) && !defined(__XBOXSERIES__)*/
 
 #endif /* SDL_VIDEO_DRIVER_WINDOWS */
 
